#!/usr.bin/python

""" Dvortype is a typing tutor for the dvorak keyboard layout
    Copyright 2004 John Bauman
    ABCD Dvorak Lessons Copyright 1995-99 Dan Wood
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    John Bauman may be contacted by email at: john@baumanfamily.com"""


import Tkinter
import CallbackEntry
import os
import time
import tkMessageBox

try:
    import winsound
    def beepnoise():
        winsound.Beep(500,100)
except:
    def beepnoise():
        root.bell()

class Lesson:
    """Implements a lesson"""
    def __init__(self, name):
        self.header = name
        self.itemlist = []
        self.length = 0
    def append(self, item):
        self.itemlist.append(item)
        self.length += len(item)
    def __getitem__(self, key):
        return self.itemlist[key]
    def __len__(self):
        return len(self.itemlist)
        
starttime = 0
savename = "prog.dat"
letters = 0
def startprogram(inputname):
    global starttime, cless, cline, errors, length, letters, entry, heading, line, root
    loadFile(inputname)
    
    if os.path.exists(savename):
        savefile = open(savename)
        cless = int(savefile.readline())
        cline = int(savefile.readline())
        length = int(savefile.readline())
        errors = int(savefile.readline())
        savefile.close()
    else:
        cless = 0
        cline = 0
        length = 0
        errors = 0
        
     
    root = Tkinter.Tk()
    root.title("ABCD Dvorak - %d/%d %d/%d" % (cless,
                                              len(lesson),
                                              cline,
                                              len(lesson[cless])))
    heading = Tkinter.Label(root, text=lesson[cless].header)
    heading.pack()
    line = Tkinter.Label(root, text=lesson[cless][cline])
    line.pack(anchor=Tkinter.W)
    
    entry = CallbackEntry.CallbackEntry(root)
    entry.callback = entryChanged
    entry.pack(expand=Tkinter.YES, fill=Tkinter.X)
    entry.focus_set()
    
    keyboardimage = Tkinter.PhotoImage(file="keycaps.gif")
    Tkinter.Label(root,image=keyboardimage).pack()

    root.mainloop()
    
    if os.path.exists(savename):
        os.remove(savename)
    fileout = open(savename, "w")
    fileout.write(str(cless) + "\n")
    fileout.write(str(cline) + "\n")
    fileout.write(str(int(length)) + "\n")
    fileout.write(str(int(errors)) + "\n")
    fileout.close()

def loadFile(inputname):
    global lesson
    lesson = []
    inputfile = open(inputname)
    for lessonn in inputfile:
        lessheader = lessonn.strip()
        less = Lesson(lessheader)
        for itemlistg in inputfile:
            if itemlistg == "%%%\n":
                break
            less.append(itemlistg.strip())      
            
        lesson.append(less)
    inputfile.close()
    
def entryChanged(newval):
    global starttime, cless, cline, errors, length
    if starttime == 0 and len(newval) != 0:
        starttime = time.time()
       
    if not lesson[cless][cline].startswith(newval):
        beepnoise()
        errors += 1
        entry.stringvar.set(newval[:-1])
        return

    if lesson[cless][cline] == newval:
        length += time.time() - starttime
        starttime = 0
        cline += 1
        letters = lesson[cless].length
        if cline >= len(lesson[cless]):
            cless += 1
            cline = 0
            lpm = letters * 60 / length
            errorp = errors * 100 / letters 
            
            if (errorp >= 12) or ( lpm < 50):
                tkMessageBox.showerror("Lesson error","""Time: %d seconds
Lpm:  %d
Errors:  %d
Error Percentage:  %d
Your error percentage was too high; you'll have to redo the lesson.""" %
                   (length, lpm, errors, errorp))
                cless -= 1
            elif cless == len(lesson):
                tkMessageBox.showinfo("Course finished","""Time: %d seconds
Lpm:  %d
Errors:  %d
Error Percentage:  %d
Wow, you finished this typing course!
It's recommended that you start over
to make sure you remember everything""" %
                   (length, lpm, errors, errorp))
                os.remove(savename)
            else:
                tkMessageBox.showinfo("Lesson progress","""Time: %d seconds
Lpm:  %d
Errors:  %d
Error Percentage:  %d""" %
                   (length, lpm, errors, errorp))
            length = 0
            errors = 0
            heading.configure(text=lesson[cless].header)
            
        entry.stringvar.set("")
        line.configure(text=lesson[cless][cline])
        root.title("ABCD Dvorak - %d/%d %d/%d" % (cless,
                                              len(lesson),
                                              cline,
                                              len(lesson[cless])))



startprogram("dvless.txt")